### ms-w32t.idl
"""
RPC definitions for the following interfaces:
- W32Time (v4.1): 8fb6d884-2388-11d0-8c35-00c04fda2795
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfPacketListField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRSignedIntField,
    NDRSignedLongField,
    NDRUnionField,
    register_dcerpc_interface,
)


class W32TimeSync_Request(NDRPacket):
    fields_desc = [NDRIntField("uWait", 0), NDRIntField("ulFlags", 0)]


class W32TimeSync_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class W32TimeGetNetlogonServiceBits_Request(NDRPacket):
    fields_desc = []


class W32TimeGetNetlogonServiceBits_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class PW32TIME_NTP_PEER_INFO(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRIntField("ulResolveAttempts", 0),
        NDRLongField("u64TimeRemaining", 0),
        NDRLongField("u64LastSuccessfulSync", 0),
        NDRIntField("ulLastSyncError", 0),
        NDRIntField("ulLastSyncErrorMsgId", 0),
        NDRIntField("ulValidDataCounter", 0),
        NDRIntField("ulAuthTypeMsgId", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wszUniqueName", ""), deferred=True
        ),
        NDRByteField("ulMode", 0),
        NDRByteField("ulStratum", 0),
        NDRByteField("ulReachability", 0),
        NDRByteField("ulPeerPollInterval", 0),
        NDRByteField("ulHostPollInterval", 0),
    ]


class W32TIME_NTP_PROVIDER_DATA(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRIntField("ulError", 0),
        NDRIntField("ulErrorMsgId", 0),
        NDRIntField("cPeerInfo", None, size_of="pPeerInfo"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pPeerInfo",
                [PW32TIME_NTP_PEER_INFO()],
                PW32TIME_NTP_PEER_INFO,
                size_is=lambda pkt: pkt.cPeerInfo,
            ),
            deferred=True,
        ),
    ]


class W32TIME_HARDWARE_PROVIDER_DATA(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRIntField("ulError", 0),
        NDRIntField("ulErrorMsgId", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wszReferenceIdentifier", ""), deferred=True
        ),
    ]


class PW32TIME_PROVIDER_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulProviderType", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "ProviderData",
                            W32TIME_NTP_PROVIDER_DATA(),
                            W32TIME_NTP_PROVIDER_DATA,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "ulProviderType", None) == 0),
                        (lambda _, val: val.tag == 0),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "ProviderData",
                            W32TIME_HARDWARE_PROVIDER_DATA(),
                            W32TIME_HARDWARE_PROVIDER_DATA,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "ulProviderType", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
            ],
            StrFixedLenField("ProviderData", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class W32TimeQueryProviderStatus_Request(NDRPacket):
    fields_desc = [
        NDRIntField("ulFlags", 0),
        NDRConfVarStrNullFieldUtf16("pwszProvider", ""),
    ]


class W32TimeQueryProviderStatus_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pProviderInfo", PW32TIME_PROVIDER_INFO(), PW32TIME_PROVIDER_INFO
            )
        ),
        NDRIntField("status", 0),
    ]


class W32TimeQuerySource_Request(NDRPacket):
    fields_desc = []


class W32TimeQuerySource_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pwszSource", "")),
        NDRIntField("status", 0),
    ]


class PW32TIME_ENTRY(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("wszName", ""), deferred=True),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("wszValue", ""), deferred=True),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("wszHelp", ""), deferred=True),
    ]


class PW32TIME_NTPCLIENT_PROVIDER_CONFIG_DATA(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRIntField("ulAllowNonstandardModeCombinations", 0),
        NDRIntField("ulCrossSiteSyncFlags", 0),
        NDRIntField("ulResolvePeerBackoffMinutes", 0),
        NDRIntField("ulResolvePeerBackoffMaxTimes", 0),
        NDRIntField("ulCompatibilityFlags", 0),
        NDRIntField("ulEventLogFlags", 0),
        NDRIntField("ulLargeSampleSkew", 0),
        NDRIntField("ulSpecialPollInterval", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("wszType", ""), deferred=True),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wszNtpServer", ""), deferred=True
        ),
        NDRIntField("ulAllowNonstandardModeCombinationsFlag", 0),
        NDRIntField("ulCrossSiteSyncFlagsFlag", 0),
        NDRIntField("ulResolvePeerBackoffMinutesFlag", 0),
        NDRIntField("ulResolvePeerBackoffMaxTimesFlag", 0),
        NDRIntField("ulCompatibilityFlagsFlag", 0),
        NDRIntField("ulEventLogFlagsFlag", 0),
        NDRIntField("ulLargeSampleSkewFlag", 0),
        NDRIntField("ulSpecialPollIntervalFlag", 0),
        NDRIntField("ulTypeFlag", 0),
        NDRIntField("ulNtpServerFlag", 0),
        NDRIntField("cEntries", None, size_of="pEntries"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pEntries",
                [PW32TIME_ENTRY()],
                PW32TIME_ENTRY,
                size_is=lambda pkt: pkt.cEntries,
            ),
            deferred=True,
        ),
    ]


class PW32TIME_NTPSERVER_PROVIDER_CONFIG_DATA(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRIntField("ulAllowNonstandardModeCombinations", 0),
        NDRIntField("ulAllowNonstandardModeCombinationsFlag", 0),
        NDRIntField("ulEventLogFlags", 0),
        NDRIntField("ulEventLogFlagsFlag", 0),
        NDRIntField("cEntries", None, size_of="pEntries"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pEntries",
                [PW32TIME_ENTRY()],
                PW32TIME_ENTRY,
                size_is=lambda pkt: pkt.cEntries,
            ),
            deferred=True,
        ),
    ]


class PW32TIME_PROVIDER_CONFIG(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRIntField("ulProviderType", 0),
        NDRFullPointerField(
            NDRUnionField(
                [
                    (
                        NDRFullPointerField(
                            NDRPacketField(
                                "pProviderConfigData",
                                PW32TIME_NTPCLIENT_PROVIDER_CONFIG_DATA(),
                                PW32TIME_NTPCLIENT_PROVIDER_CONFIG_DATA,
                            ),
                            deferred=True,
                        ),
                        (
                            (lambda pkt: getattr(pkt, "ulProviderType", None) == 0),
                            (lambda _, val: val.tag == 0),
                        ),
                    ),
                    (
                        NDRFullPointerField(
                            NDRPacketField(
                                "pProviderConfigData",
                                PW32TIME_NTPSERVER_PROVIDER_CONFIG_DATA(),
                                PW32TIME_NTPSERVER_PROVIDER_CONFIG_DATA,
                            ),
                            deferred=True,
                        ),
                        (
                            (lambda pkt: getattr(pkt, "ulProviderType", None) == 1),
                            (lambda _, val: val.tag == 1),
                        ),
                    ),
                ],
                StrFixedLenField("pProviderConfigData", "", length=0),
                align=(4, 8),
                switch_fmt=("L", "L"),
            ),
            deferred=True,
        ),
    ]


class PW32TIME_CONFIGURATION_PROVIDER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRIntField("ulInputProvider", 0),
        NDRIntField("ulEnabled", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wszDllName", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wszProviderName", ""), deferred=True
        ),
        NDRIntField("ulDllNameFlag", 0),
        NDRIntField("ulProviderNameFlag", 0),
        NDRIntField("ulInputProviderFlag", 0),
        NDRIntField("ulEnabledFlag", 0),
        NDRFullPointerField(
            NDRPacketField(
                "pProviderConfig", PW32TIME_PROVIDER_CONFIG(), PW32TIME_PROVIDER_CONFIG
            ),
            deferred=True,
        ),
    ]


class W32TimeQueryProviderConfiguration_Request(NDRPacket):
    fields_desc = [
        NDRIntField("ulFlags", 0),
        NDRConfVarStrNullFieldUtf16("pwszProvider", ""),
    ]


class W32TimeQueryProviderConfiguration_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pConfigurationProviderInfo",
                PW32TIME_CONFIGURATION_PROVIDER(),
                PW32TIME_CONFIGURATION_PROVIDER,
            )
        ),
        NDRIntField("status", 0),
    ]


class W32TIME_CONFIGURATION_BASIC(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRIntField("ulEventLogFlags", 0),
        NDRIntField("ulAnnounceFlags", 0),
        NDRIntField("ulTimeJumpAuditOffset", 0),
        NDRIntField("ulMinPollInterval", 0),
        NDRIntField("ulMaxPollInterval", 0),
        NDRIntField("ulMaxNegPhaseCorrection", 0),
        NDRIntField("ulMaxPosPhaseCorrection", 0),
        NDRIntField("ulMaxAllowedPhaseOffset", 0),
        NDRIntField("ulEventLogFlagsFlag", 0),
        NDRIntField("ulAnnounceFlagsFlag", 0),
        NDRIntField("ulTimeJumpAuditOffsetFlag", 0),
        NDRIntField("ulMinPollIntervalFlag", 0),
        NDRIntField("ulMaxPollIntervalFlag", 0),
        NDRIntField("ulMaxNegPhaseCorrectionFlag", 0),
        NDRIntField("ulMaxPosPhaseCorrectionFlag", 0),
        NDRIntField("ulMaxAllowedPhaseOffsetFlag", 0),
    ]


class W32TIME_CONFIGURATION_ADVANCED(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRIntField("ulFrequencyCorrectRate", 0),
        NDRIntField("ulPollAdjustFactor", 0),
        NDRIntField("ulLargePhaseOffset", 0),
        NDRIntField("ulSpikeWatchPeriod", 0),
        NDRIntField("ulLocalClockDispersion", 0),
        NDRIntField("ulHoldPeriod", 0),
        NDRIntField("ulPhaseCorrectRate", 0),
        NDRIntField("ulUpdateInterval", 0),
        NDRIntField("ulFrequencyCorrectRateFlag", 0),
        NDRIntField("ulPollAdjustFactorFlag", 0),
        NDRIntField("ulLargePhaseOffsetFlag", 0),
        NDRIntField("ulSpikeWatchPeriodFlag", 0),
        NDRIntField("ulLocalClockDispersionFlag", 0),
        NDRIntField("ulHoldPeriodFlag", 0),
        NDRIntField("ulPhaseCorrectRateFlag", 0),
        NDRIntField("ulUpdateIntervalFlag", 0),
    ]


class W32TIME_CONFIGURATION_DEFAULT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wszFileLogName", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wszFileLogEntries", ""), deferred=True
        ),
        NDRIntField("ulFileLogSize", 0),
        NDRIntField("ulFileLogFlags", 0),
        NDRIntField("ulFileLogNameFlag", 0),
        NDRIntField("ulFileLogEntriesFlag", 0),
        NDRIntField("ulFileLogSizeFlag", 0),
        NDRIntField("ulFileLogFlagsFlag", 0),
    ]


class PW32TIME_CONFIGURATION_INFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRPacketField(
            "basicConfig", W32TIME_CONFIGURATION_BASIC(), W32TIME_CONFIGURATION_BASIC
        ),
        NDRPacketField(
            "advancedConfig",
            W32TIME_CONFIGURATION_ADVANCED(),
            W32TIME_CONFIGURATION_ADVANCED,
        ),
        NDRPacketField(
            "defaultConfig",
            W32TIME_CONFIGURATION_DEFAULT(),
            W32TIME_CONFIGURATION_DEFAULT,
        ),
        NDRIntField("cProviderConfig", None, size_of="pProviderConfig"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pProviderConfig",
                [],
                PW32TIME_CONFIGURATION_PROVIDER,
                size_is=lambda pkt: pkt.cProviderConfig,
                ptr_pack=True,
            ),
            deferred=True,
        ),
        NDRIntField("cEntries", None, size_of="pEntries"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pEntries",
                [PW32TIME_ENTRY()],
                PW32TIME_ENTRY,
                size_is=lambda pkt: pkt.cEntries,
            ),
            deferred=True,
        ),
    ]


class W32TimeQueryConfiguration_Request(NDRPacket):
    fields_desc = []


class W32TimeQueryConfiguration_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pConfigurationInfo",
                PW32TIME_CONFIGURATION_INFO(),
                PW32TIME_CONFIGURATION_INFO,
            )
        ),
        NDRIntField("status", 0),
    ]


class PW32TIME_STATUS_INFO(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRIntField("ulSize", 0),
        NDRIntField("eLeapIndicator", 0),
        NDRIntField("nStratum", 0),
        NDRSignedIntField("nPollInterval", 0),
        NDRIntField("refidSource", 0),
        NDRLongField("qwLastSyncTicks", 0),
        NDRSignedLongField("toRootDelay", 0),
        NDRLongField("tpRootDispersion", 0),
        NDRSignedIntField("nClockPrecision", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wszSource", ""), deferred=True
        ),
        NDRSignedLongField("toSysPhaseOffset", 0),
        NDRIntField("ulLcState", 0),
        NDRIntField("ulTSFlags", 0),
        NDRIntField("ulClockRate", 0),
        NDRIntField("ulNetlogonServiceBits", 0),
        NDRIntField("eLastSyncResult", 0),
        NDRLongField("tpTimeLastGoodSync", 0),
        NDRIntField("cEntries", None, size_of="pEntries"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "pEntries",
                [PW32TIME_ENTRY()],
                PW32TIME_ENTRY,
                size_is=lambda pkt: pkt.cEntries,
            ),
            deferred=True,
        ),
    ]


class W32TimeQueryStatus_Request(NDRPacket):
    fields_desc = []


class W32TimeQueryStatus_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pStatusInfo", PW32TIME_STATUS_INFO(), PW32TIME_STATUS_INFO)
        ),
        NDRIntField("status", 0),
    ]


class W32TimeLog_Request(NDRPacket):
    fields_desc = []


class W32TimeLog_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


W32TIME_OPNUMS = {
    0: DceRpcOp(W32TimeSync_Request, W32TimeSync_Response),
    1: DceRpcOp(
        W32TimeGetNetlogonServiceBits_Request, W32TimeGetNetlogonServiceBits_Response
    ),
    2: DceRpcOp(
        W32TimeQueryProviderStatus_Request, W32TimeQueryProviderStatus_Response
    ),
    3: DceRpcOp(W32TimeQuerySource_Request, W32TimeQuerySource_Response),
    4: DceRpcOp(
        W32TimeQueryProviderConfiguration_Request,
        W32TimeQueryProviderConfiguration_Response,
    ),
    5: DceRpcOp(W32TimeQueryConfiguration_Request, W32TimeQueryConfiguration_Response),
    6: DceRpcOp(W32TimeQueryStatus_Request, W32TimeQueryStatus_Response),
    7: DceRpcOp(W32TimeLog_Request, W32TimeLog_Response),
}
register_dcerpc_interface(
    name="W32Time",
    uuid=uuid.UUID("8fb6d884-2388-11d0-8c35-00c04fda2795"),
    version="4.1",
    opnums=W32TIME_OPNUMS,
)



### ms-brwsa.idl
"""
RPC definitions for the following interfaces:
- browser (v0.0): 6BFFD098-A112-3610-9833-012892020162
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRUnionField,
    register_dcerpc_interface,
)


class LPSERVER_INFO_100(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("sv100_platform_id", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("sv100_name", ""), deferred=True
        ),
    ]


class LPSERVER_INFO_100_CONTAINER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("EntriesRead", None, size_of="Buffer"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Buffer",
                [LPSERVER_INFO_100()],
                LPSERVER_INFO_100,
                size_is=lambda pkt: pkt.EntriesRead,
            ),
            deferred=True,
        ),
    ]


class LPSERVER_ENUM_STRUCT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "ServerInfo",
                            LPSERVER_INFO_100_CONTAINER(),
                            LPSERVER_INFO_100_CONTAINER,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 100),
                        (lambda _, val: val.tag == 100),
                    ),
                )
            ],
            StrFixedLenField("ServerInfo", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class I_BrowserrQueryOtherDomains_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRPacketField("InfoStruct", LPSERVER_ENUM_STRUCT(), LPSERVER_ENUM_STRUCT),
    ]


class I_BrowserrQueryOtherDomains_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("InfoStruct", LPSERVER_ENUM_STRUCT(), LPSERVER_ENUM_STRUCT),
        NDRIntField("TotalEntries", 0),
        NDRIntField("status", 0),
    ]


BROWSER_OPNUMS = {  # 0: Opnum0NotUsedOnWire,
    # 1: Opnum1NotUsedOnWire,
    2: DceRpcOp(
        I_BrowserrQueryOtherDomains_Request, I_BrowserrQueryOtherDomains_Response
    ),
    # 3: Opnum3NotUsedOnWire,
    # 4: Opnum4NotUsedOnWire,
    # 5: Opnum5NotUsedOnWire,
    # 6: Opnum6NotUsedOnWire,
    # 7: Opnum7NotUsedOnWire,
    # 8: Opnum8NotUsedOnWire,
    # 9: Opnum9NotUsedOnWire,
    # 10: Opnum10NotUsedOnWire,
    # 11: Opnum11NotUsedOnWire
}
register_dcerpc_interface(
    name="browser",
    uuid=uuid.UUID("6BFFD098-A112-3610-9833-012892020162"),
    version="0.0",
    opnums=BROWSER_OPNUMS,
)



### ms-iiss.idl
"""
RPC definitions for the following interfaces:
- IIisServiceControl (v0.0): E8FB8620-588F-11D2-9D61-00C04F79C5FE
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfStrLenField,
    NDRIntField,
    register_com_interface,
)


class Stop_Request(NDRPacket):
    fields_desc = []


class Stop_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Start_Request(NDRPacket):
    fields_desc = []


class Start_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Reboot_Request(NDRPacket):
    fields_desc = []


class Reboot_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Status_Request(NDRPacket):
    fields_desc = [NDRIntField("dwBufferSize", 0)]


class Status_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("pbBuffer", "", size_is=lambda pkt: pkt.dwBufferSize),
        NDRIntField("pdwMDRequiredBufferSize", 0),
        NDRIntField("pdwNumServices", 0),
        NDRIntField("status", 0),
    ]


class Kill_Request(NDRPacket):
    fields_desc = []


class Kill_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IIISSERVICECONTROL_OPNUMS = {
    0: DceRpcOp(Stop_Request, Stop_Response),
    1: DceRpcOp(Start_Request, Start_Response),
    2: DceRpcOp(Reboot_Request, Reboot_Response),
    3: DceRpcOp(Status_Request, Status_Response),
    4: DceRpcOp(Kill_Request, Kill_Response),
}
register_com_interface(
    name="IIisServiceControl",
    uuid=uuid.UUID("E8FB8620-588F-11D2-9D61-00C04F79C5FE"),
    opnums=IIISSERVICECONTROL_OPNUMS,
)



### ms-dmrp_dmintf3.idl
"""
RPC definitions for the following interfaces:
- IVolumeClient3 (v0.0): 135698D2-3A37-4d26-99DF-E2BB6AE3AC61
- IVolumeClient4 (v0.0): DEB01010-3A37-4d26-99DF-E2BB6AE3AC61
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField, StrFixedLenFieldUtf16
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    NDRUnionField,
    register_com_interface,
)


class PARTITIONSTYLE(IntEnum):
    PARTITIONSTYLE_UNKNOWN = 0
    PARTITIONSTYLE_MBR = 1
    PARTITIONSTYLE_GPT = 2


class mbr_sub0(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("signature", 0)]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class gpt_sub1(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRPacketField("diskId", GUID(), GUID)]


class diskinfoex(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRSignedLongField("id", 0),
        NDRSignedLongField("length", 0),
        NDRSignedLongField("freeBytes", 0),
        NDRIntField("bytesPerTrack", 0),
        NDRIntField("bytesPerCylinder", 0),
        NDRIntField("bytesPerSector", 0),
        NDRIntField("regionCount", 0),
        NDRIntField("dflags", 0),
        NDRIntField("deviceType", 0),
        NDRIntField("deviceState", 0),
        NDRIntField("busType", 0),
        NDRIntField("attributes", 0),
        NDRIntField("maxPartitionCount", 0),
        NDRIntField("isUpgradeable", 0),
        NDRIntField("maySwitchStyle", 0),
        NDRInt3264EnumField("partitionStyle", 0, PARTITIONSTYLE),
        NDRUnionField(
            [
                (
                    NDRPacketField("value", mbr_sub0(), mbr_sub0),
                    (
                        (
                            lambda pkt: getattr(pkt, "partitionStyle", None)
                            == PARTITIONSTYLE.PARTITIONSTYLE_MBR
                        ),
                        (lambda _, val: val.tag == PARTITIONSTYLE.PARTITIONSTYLE_MBR),
                    ),
                ),
                (
                    NDRPacketField("value", gpt_sub1(), gpt_sub1),
                    (
                        (
                            lambda pkt: getattr(pkt, "partitionStyle", None)
                            == PARTITIONSTYLE.PARTITIONSTYLE_GPT
                        ),
                        (lambda _, val: val.tag == PARTITIONSTYLE.PARTITIONSTYLE_GPT),
                    ),
                ),
            ],
            StrFixedLenField("value", "", length=0),
            align=(2, 4),
            switch_fmt=("H", "I"),
        ),
        NDRSignedIntField("portNumber", 0),
        NDRSignedIntField("targetNumber", 0),
        NDRSignedIntField("lunNumber", 0),
        NDRSignedLongField("lastKnownState", 0),
        NDRSignedLongField("taskId", 0),
        NDRSignedIntField("cchName", None, size_of="name"),
        NDRSignedIntField("cchVendor", None, size_of="vendor"),
        NDRSignedIntField("cchDgid", None, size_of="dgid"),
        NDRSignedIntField("cchAdapterName", None, size_of="adapterName"),
        NDRSignedIntField("cchDgName", None, size_of="dgName"),
        NDRSignedIntField("cchDevInstId", None, size_of="devInstId"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("name", "", size_is=lambda pkt: pkt.cchName),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("vendor", "", size_is=lambda pkt: pkt.cchVendor),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRConfStrLenField("dgid", "", size_is=lambda pkt: pkt.cchDgid),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16(
                "adapterName", "", size_is=lambda pkt: pkt.cchAdapterName
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("dgName", "", size_is=lambda pkt: pkt.cchDgName),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16(
                "devInstId", "", size_is=lambda pkt: pkt.cchDevInstId
            ),
            deferred=True,
        ),
    ]


class EnumDisksEx_Request(NDRPacket):
    fields_desc = []


class EnumDisksEx_Response(NDRPacket):
    fields_desc = [
        NDRIntField("diskCount", None, size_of="diskList"),
        NDRConfPacketListField(
            "diskList", [], diskinfoex, size_is=lambda pkt: pkt.diskCount, ptr_pack=True
        ),
        NDRIntField("status", 0),
    ]


class REGIONTYPE(IntEnum):
    REGION_UNKNOWN = 1
    REGION_FREE = 2
    REGION_EXTENDED_FREE = 3
    REGION_PRIMARY = 4
    REGION_LOGICAL = 5
    REGION_EXTENDED = 6
    REGION_SUBDISK = 7
    REGION_CDROM = 8
    REGION_REMOVABLE = 9


class mbr_sub0(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("partitionType", 0), NDRIntField("isActive", 0)]


class gpt_sub1(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRPacketField("partitionType", GUID(), GUID),
        NDRPacketField("partitionId", GUID(), GUID),
        NDRLongField("attributes", 0),
    ]


class REGIONSTATUS(IntEnum):
    REGIONSTATUS_UNKNOWN = 1
    REGIONSTATUS_OK = 2
    REGIONSTATUS_FAILED = 3
    REGIONSTATUS_FAILING = 4
    REGIONSTATUS_REGENERATING = 5
    REGIONSTATUS_NEEDSRESYNC = 6


class regioninfoex(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRSignedLongField("id", 0),
        NDRSignedLongField("diskId", 0),
        NDRSignedLongField("volId", 0),
        NDRSignedLongField("fsId", 0),
        NDRSignedLongField("start", 0),
        NDRSignedLongField("length", 0),
        NDRInt3264EnumField("regionType", 0, REGIONTYPE),
        NDRInt3264EnumField("partitionStyle", 0, PARTITIONSTYLE),
        NDRUnionField(
            [
                (
                    NDRPacketField("value", mbr_sub0(), mbr_sub0),
                    (
                        (
                            lambda pkt: getattr(pkt, "partitionStyle", None)
                            == PARTITIONSTYLE.PARTITIONSTYLE_MBR
                        ),
                        (lambda _, val: val.tag == PARTITIONSTYLE.PARTITIONSTYLE_MBR),
                    ),
                ),
                (
                    NDRPacketField("value", gpt_sub1(), gpt_sub1),
                    (
                        (
                            lambda pkt: getattr(pkt, "partitionStyle", None)
                            == PARTITIONSTYLE.PARTITIONSTYLE_GPT
                        ),
                        (lambda _, val: val.tag == PARTITIONSTYLE.PARTITIONSTYLE_GPT),
                    ),
                ),
            ],
            StrFixedLenField("value", "", length=0),
            align=(2, 8),
            switch_fmt=("H", "I"),
        ),
        NDRInt3264EnumField("status", 0, REGIONSTATUS),
        NDRSignedLongField("lastKnownState", 0),
        NDRSignedLongField("taskId", 0),
        NDRIntField("rflags", 0),
        NDRIntField("currentPartitionNumber", 0),
        NDRSignedIntField("cchName", None, size_of="name"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("name", "", size_is=lambda pkt: pkt.cchName),
            deferred=True,
        ),
    ]


class EnumDiskRegionsEx_Request(NDRPacket):
    fields_desc = [NDRSignedLongField("diskId", 0), NDRIntField("numRegions", 0)]


class EnumDiskRegionsEx_Response(NDRPacket):
    fields_desc = [
        NDRIntField("numRegions", None, size_of="regionList"),
        NDRConfPacketListField(
            "regionList",
            [],
            regioninfoex,
            size_is=lambda pkt: pkt.numRegions,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class regionspec(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRSignedLongField("regionId", 0),
        NDRInt3264EnumField("regionType", 0, REGIONTYPE),
        NDRSignedLongField("diskId", 0),
        NDRSignedLongField("start", 0),
        NDRSignedLongField("length", 0),
        NDRSignedLongField("lastKnownState", 0),
    ]


class REQSTATUS(IntEnum):
    REQ_UNKNOWN = 1
    REQ_STARTED = 2
    REQ_IN_PROGRESS = 3
    REQ_COMPLETED = 4
    REQ_ABORTED = 5
    REQ_FAILED = 6


class DMPROGRESS_TYPE(IntEnum):
    PROGRESS_UNKNOWN = 1
    PROGRESS_FORMAT = 2
    PROGRESS_SYNCHING = 3


class taskinfo(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRSignedLongField("id", 0),
        NDRSignedLongField("storageId", 0),
        NDRSignedLongField("createTime", 0),
        NDRSignedLongField("clientID", 0),
        NDRIntField("percentComplete", 0),
        NDRInt3264EnumField("status", 0, REQSTATUS),
        NDRInt3264EnumField("type", 0, DMPROGRESS_TYPE),
        NDRSignedIntField("error", 0),
        NDRIntField("tflag", 0),
    ]


class CreatePartition_Request(NDRPacket):
    fields_desc = [NDRPacketField("partitionSpec", regionspec(), regionspec)]


class CreatePartition_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class filesysteminfo(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRSignedLongField("id", 0),
        NDRSignedLongField("storageId", 0),
        NDRSignedLongField("totalAllocationUnits", 0),
        NDRSignedLongField("availableAllocationUnits", 0),
        NDRIntField("allocationUnitSize", 0),
        NDRIntField("fsflags", 0),
        NDRSignedLongField("lastKnownState", 0),
        NDRSignedLongField("taskId", 0),
        NDRSignedIntField("fsType", 0),
        NDRSignedIntField("cchLabel", None, size_of="label"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("label", "", size_is=lambda pkt: pkt.cchLabel),
            deferred=True,
        ),
    ]


class CreatePartitionAssignAndFormat_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("partitionSpec", regionspec(), regionspec),
        NDRShortField("letter", 0),
        NDRSignedLongField("letterLastKnownState", 0),
        NDRPacketField("fsSpec", filesysteminfo(), filesysteminfo),
        NDRIntField("quickFormat", 0),
    ]


class CreatePartitionAssignAndFormat_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class CreatePartitionAssignAndFormatEx_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("partitionSpec", regionspec(), regionspec),
        NDRShortField("letter", 0),
        NDRSignedLongField("letterLastKnownState", 0),
        NDRSignedIntField("cchAccessPath", None, size_of="AccessPath"),
        NDRConfStrLenFieldUtf16(
            "AccessPath", "", size_is=lambda pkt: pkt.cchAccessPath
        ),
        NDRPacketField("fsSpec", filesysteminfo(), filesysteminfo),
        NDRIntField("quickFormat", 0),
        NDRIntField("dwFlags", 0),
    ]


class CreatePartitionAssignAndFormatEx_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class DeletePartition_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("partitionSpec", regionspec(), regionspec),
        NDRIntField("force", 0),
    ]


class DeletePartition_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class InitializeDiskStyle_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("diskId", 0),
        NDRInt3264EnumField("style", 0, PARTITIONSTYLE),
        NDRSignedLongField("diskLastKnownState", 0),
    ]


class InitializeDiskStyle_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class MarkActivePartition_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("regionId", 0),
        NDRSignedLongField("regionLastKnownState", 0),
    ]


class MarkActivePartition_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class Eject_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("diskId", 0),
        NDRSignedLongField("diskLastKnownState", 0),
    ]


class Eject_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class Reserved_Opnum12_Request(NDRPacket):
    fields_desc = []


class Reserved_Opnum12_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class VOLUMETYPE(IntEnum):
    VOLUMETYPE_UNKNOWN = 1
    VOLUMETYPE_PRIMARY_PARTITION = 2
    VOLUMETYPE_LOGICAL_DRIVE = 3
    VOLUMETYPE_FT = 4
    VOLUMETYPE_VM = 5
    VOLUMETYPE_CDROM = 6
    VOLUMETYPE_REMOVABLE = 7


class VOLUMELAYOUT(IntEnum):
    VOLUMELAYOUT_UNKNOWN = 1
    VOLUMELAYOUT_PARTITION = 2
    VOLUMELAYOUT_SIMPLE = 3
    VOLUMELAYOUT_SPANNED = 4
    VOLUMELAYOUT_MIRROR = 5
    VOLUMELAYOUT_STRIPE = 6
    VOLUMELAYOUT_RAID5 = 7


class VOLUMESTATUS(IntEnum):
    VOLUME_STATUS_UNKNOWN = 1
    VOLUME_STATUS_HEALTHY = 2
    VOLUME_STATUS_FAILED = 3
    VOLUME_STATUS_FAILED_REDUNDANCY = 4
    VOLUME_STATUS_FAILING = 5
    VOLUME_STATUS_FAILING_REDUNDANCY = 6
    VOLUME_STATUS_FAILED_REDUNDANCY_FAILING = 7
    VOLUME_STATUS_SYNCHING = 8
    VOLUME_STATUS_REGENERATING = 9
    VOLUME_STATUS_INITIALIZING = 10
    VOLUME_STATUS_FORMATTING = 11


class VOLUME_INFO(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRSignedLongField("id", 0),
        NDRInt3264EnumField("type", 0, VOLUMETYPE),
        NDRInt3264EnumField("layout", 0, VOLUMELAYOUT),
        NDRSignedLongField("length", 0),
        NDRSignedLongField("fsId", 0),
        NDRIntField("memberCount", 0),
        NDRInt3264EnumField("status", 0, VOLUMESTATUS),
        NDRSignedLongField("lastKnownState", 0),
        NDRSignedLongField("taskId", 0),
        NDRIntField("vflags", 0),
    ]


class FTEnumVolumes_Request(NDRPacket):
    fields_desc = [NDRIntField("volumeCount", 0)]


class FTEnumVolumes_Response(NDRPacket):
    fields_desc = [
        NDRIntField("volumeCount", None, size_of="ftVolumeList"),
        NDRConfPacketListField(
            "ftVolumeList",
            [],
            VOLUME_INFO,
            size_is=lambda pkt: pkt.volumeCount,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class FTEnumLogicalDiskMembers_Request(NDRPacket):
    fields_desc = [NDRSignedLongField("volumeId", 0), NDRIntField("memberCount", 0)]


class FTEnumLogicalDiskMembers_Response(NDRPacket):
    fields_desc = [
        NDRIntField("memberCount", None, size_of="memberList"),
        NDRConfFieldListField(
            "memberList",
            [],
            NDRSignedLongField("", 0),
            size_is=lambda pkt: pkt.memberCount,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class FTDeleteVolume_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRIntField("force", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
    ]


class FTDeleteVolume_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class FTBreakMirror_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
        NDRIntField("bForce", 0),
    ]


class FTBreakMirror_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class FTResyncMirror_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
    ]


class FTResyncMirror_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class FTRegenerateParityStripe_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
    ]


class FTRegenerateParityStripe_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class FTReplaceMirrorPartition_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
        NDRSignedLongField("oldMemberId", 0),
        NDRSignedLongField("oldMemberLastKnownState", 0),
        NDRSignedLongField("newRegionId", 0),
        NDRSignedLongField("newRegionLastKnownState", 0),
        NDRIntField("flags", 0),
    ]


class FTReplaceMirrorPartition_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class FTReplaceParityStripePartition_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
        NDRSignedLongField("oldMemberId", 0),
        NDRSignedLongField("oldMemberLastKnownState", 0),
        NDRSignedLongField("newRegionId", 0),
        NDRSignedLongField("newRegionLastKnownState", 0),
        NDRIntField("flags", 0),
    ]


class FTReplaceParityStripePartition_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class driveletterinfo(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRShortField("letter", 0),
        NDRSignedLongField("storageId", 0),
        NDRIntField("isUsed", 0),
        NDRSignedLongField("lastKnownState", 0),
        NDRSignedLongField("taskId", 0),
        NDRIntField("dlflags", 0),
    ]


class EnumDriveLetters_Request(NDRPacket):
    fields_desc = [NDRIntField("driveLetterCount", 0)]


class EnumDriveLetters_Response(NDRPacket):
    fields_desc = [
        NDRIntField("driveLetterCount", None, size_of="driveLetterList"),
        NDRConfPacketListField(
            "driveLetterList",
            [],
            driveletterinfo,
            size_is=lambda pkt: pkt.driveLetterCount,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class AssignDriveLetter_Request(NDRPacket):
    fields_desc = [
        NDRShortField("letter", 0),
        NDRIntField("forceOption", 0),
        NDRSignedLongField("letterLastKnownState", 0),
        NDRSignedLongField("storageId", 0),
        NDRSignedLongField("storageLastKnownState", 0),
    ]


class AssignDriveLetter_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class FreeDriveLetter_Request(NDRPacket):
    fields_desc = [
        NDRShortField("letter", 0),
        NDRIntField("forceOption", 0),
        NDRSignedLongField("letterLastKnownState", 0),
        NDRSignedLongField("storageId", 0),
        NDRSignedLongField("storageLastKnownState", 0),
    ]


class FreeDriveLetter_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class EnumLocalFileSystems_Request(NDRPacket):
    fields_desc = []


class EnumLocalFileSystems_Response(NDRPacket):
    fields_desc = [
        NDRIntField("fileSystemCount", None, size_of="fileSystemList"),
        NDRConfPacketListField(
            "fileSystemList",
            [],
            filesysteminfo,
            size_is=lambda pkt: pkt.fileSystemCount,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class ifilesysteminfo(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRSignedIntField("fsType", 0),
        StrFixedLenFieldUtf16("fsName", "", length=8 * 2),
        NDRIntField("fsFlags", 0),
        NDRIntField("fsCompressionFlags", 0),
        NDRSignedIntField("cchLabelLimit", 0),
        NDRSignedIntField("cchLabel", None, size_of="iLabelChSet"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16(
                "iLabelChSet", "", size_is=lambda pkt: pkt.cchLabel
            ),
            deferred=True,
        ),
    ]


class GetInstalledFileSystems_Request(NDRPacket):
    fields_desc = []


class GetInstalledFileSystems_Response(NDRPacket):
    fields_desc = [
        NDRIntField("fsCount", None, size_of="fsList"),
        NDRConfPacketListField(
            "fsList",
            [],
            ifilesysteminfo,
            size_is=lambda pkt: pkt.fsCount,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class Format_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("storageId", 0),
        NDRPacketField("fsSpec", filesysteminfo(), filesysteminfo),
        NDRIntField("quickFormat", 0),
        NDRIntField("force", 0),
        NDRSignedLongField("storageLastKnownState", 0),
    ]


class Format_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class EnumVolumes_Request(NDRPacket):
    fields_desc = [NDRIntField("volumeCount", 0)]


class EnumVolumes_Response(NDRPacket):
    fields_desc = [
        NDRIntField("volumeCount", None, size_of="LdmVolumeList"),
        NDRConfPacketListField(
            "LdmVolumeList",
            [],
            VOLUME_INFO,
            size_is=lambda pkt: pkt.volumeCount,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class EnumVolumeMembers_Request(NDRPacket):
    fields_desc = [NDRSignedLongField("volumeId", 0), NDRIntField("memberCount", 0)]


class EnumVolumeMembers_Response(NDRPacket):
    fields_desc = [
        NDRIntField("memberCount", None, size_of="memberList"),
        NDRConfFieldListField(
            "memberList",
            [],
            NDRSignedLongField("", 0),
            size_is=lambda pkt: pkt.memberCount,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


class VOLUME_SPEC(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRInt3264EnumField("type", 0, VOLUMETYPE),
        NDRInt3264EnumField("layout", 0, VOLUMELAYOUT),
        NDRInt3264EnumField("partitionType", 0, REGIONTYPE),
        NDRSignedLongField("length", 0),
        NDRSignedLongField("lastKnownState", 0),
    ]


class diskspec(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRSignedLongField("diskId", 0),
        NDRSignedLongField("length", 0),
        NDRIntField("needContiguous", 0),
        NDRSignedLongField("lastKnownState", 0),
    ]


class CreateVolume_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("volumeSpec", VOLUME_SPEC(), VOLUME_SPEC),
        NDRIntField("diskCount", None, size_of="diskList"),
        NDRConfPacketListField(
            "diskList", [], diskspec, size_is=lambda pkt: pkt.diskCount
        ),
    ]


class CreateVolume_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class CreateVolumeAssignAndFormat_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("volumeSpec", VOLUME_SPEC(), VOLUME_SPEC),
        NDRIntField("diskCount", None, size_of="diskList"),
        NDRConfPacketListField(
            "diskList", [], diskspec, size_is=lambda pkt: pkt.diskCount
        ),
        NDRShortField("letter", 0),
        NDRSignedLongField("letterLastKnownState", 0),
        NDRPacketField("fsSpec", filesysteminfo(), filesysteminfo),
        NDRIntField("quickFormat", 0),
    ]


class CreateVolumeAssignAndFormat_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class CreateVolumeAssignAndFormatEx_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("volumeSpec", VOLUME_SPEC(), VOLUME_SPEC),
        NDRIntField("diskCount", None, size_of="diskList"),
        NDRConfPacketListField(
            "diskList", [], diskspec, size_is=lambda pkt: pkt.diskCount
        ),
        NDRShortField("letter", 0),
        NDRSignedLongField("letterLastKnownState", 0),
        NDRSignedIntField("cchAccessPath", None, size_of="AccessPath"),
        NDRConfStrLenFieldUtf16(
            "AccessPath", "", size_is=lambda pkt: pkt.cchAccessPath
        ),
        NDRPacketField("fsSpec", filesysteminfo(), filesysteminfo),
        NDRIntField("quickFormat", 0),
        NDRIntField("dwFlags", 0),
    ]


class CreateVolumeAssignAndFormatEx_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class GetVolumeMountName_Request(NDRPacket):
    fields_desc = [NDRSignedLongField("volumeId", 0)]


class GetVolumeMountName_Response(NDRPacket):
    fields_desc = [
        NDRIntField("cchMountName", None, size_of="mountName"),
        NDRConfStrLenFieldUtf16("mountName", "", size_is=lambda pkt: pkt.cchMountName),
        NDRIntField("status", 0),
    ]


class GrowVolume_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRPacketField("volumeSpec", VOLUME_SPEC(), VOLUME_SPEC),
        NDRIntField("diskCount", None, size_of="diskList"),
        NDRConfPacketListField(
            "diskList", [], diskspec, size_is=lambda pkt: pkt.diskCount
        ),
        NDRIntField("force", 0),
    ]


class GrowVolume_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class DeleteVolume_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRIntField("force", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
    ]


class DeleteVolume_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class CreatePartitionsForVolume_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRIntField("active", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
    ]


class CreatePartitionsForVolume_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class DeletePartitionsForVolume_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
    ]


class DeletePartitionsForVolume_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class GetMaxAdjustedFreeSpace_Request(NDRPacket):
    fields_desc = [NDRSignedLongField("diskId", 0)]


class GetMaxAdjustedFreeSpace_Response(NDRPacket):
    fields_desc = [
        NDRSignedLongField("maxAdjustedFreeSpace", 0),
        NDRIntField("status", 0),
    ]


class AddMirror_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
        NDRPacketField("diskSpec", diskspec(), diskspec),
        NDRSignedIntField("diskNumber", 0),
    ]


class AddMirror_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("diskNumber", 0),
        NDRSignedIntField("partitionNumber", 0),
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class RemoveMirror_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
        NDRSignedLongField("diskId", 0),
        NDRSignedLongField("diskLastKnownState", 0),
    ]


class RemoveMirror_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class SplitMirror_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
        NDRSignedLongField("diskId", 0),
        NDRSignedLongField("diskLastKnownState", 0),
        NDRShortField("letter", 0),
        NDRSignedLongField("letterLastKnownState", 0),
        NDRPacketField("tinfo", taskinfo(), taskinfo),
    ]


class SplitMirror_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class InitializeDiskEx_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("diskId", 0),
        NDRInt3264EnumField("style", 0, PARTITIONSTYLE),
        NDRSignedLongField("diskLastKnownState", 0),
    ]


class InitializeDiskEx_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class UninitializeDisk_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("diskId", 0),
        NDRSignedLongField("diskLastKnownState", 0),
    ]


class UninitializeDisk_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class ReConnectDisk_Request(NDRPacket):
    fields_desc = [NDRSignedLongField("diskId", 0)]


class ReConnectDisk_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class ImportDiskGroup_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("cchDgid", None, size_of="dgid"),
        NDRConfStrLenField("dgid", "", size_is=lambda pkt: pkt.cchDgid),
    ]


class ImportDiskGroup_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class mergeobjectinfo(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRIntField("type", 0),
        NDRIntField("flags", 0),
        NDRInt3264EnumField("layout", 0, VOLUMELAYOUT),
        NDRSignedLongField("length", 0),
    ]


class DiskMergeQuery_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("cchDgid", None, size_of="dgid"),
        NDRConfStrLenField("dgid", "", size_is=lambda pkt: pkt.cchDgid),
        NDRSignedIntField("numDisks", None, size_of="diskList"),
        NDRConfFieldListField(
            "diskList", [], NDRSignedLongField("", 0), size_is=lambda pkt: pkt.numDisks
        ),
        NDRIntField("flags", 0),
    ]


class DiskMergeQuery_Response(NDRPacket):
    fields_desc = [
        NDRSignedLongField("merge_config_tid", 0),
        NDRSignedIntField("numRids", None, size_of="merge_dm_rids"),
        NDRConfFieldListField(
            "merge_dm_rids",
            [],
            NDRSignedLongField("", 0),
            size_is=lambda pkt: pkt.numRids,
            ptr_pack=True,
        ),
        NDRSignedIntField("numObjects", None, size_of="mergeObjectInfo"),
        NDRConfPacketListField(
            "mergeObjectInfo",
            [],
            mergeobjectinfo,
            size_is=lambda pkt: pkt.numObjects,
            ptr_pack=True,
        ),
        NDRIntField("flags", 0),
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class DiskMerge_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("cchDgid", None, size_of="dgid"),
        NDRConfStrLenField("dgid", "", size_is=lambda pkt: pkt.cchDgid),
        NDRSignedIntField("numDisks", None, size_of="diskList"),
        NDRConfFieldListField(
            "diskList", [], NDRSignedLongField("", 0), size_is=lambda pkt: pkt.numDisks
        ),
        NDRSignedLongField("merge_config_tid", 0),
        NDRSignedIntField("numRids", None, size_of="merge_dm_rids"),
        NDRConfFieldListField(
            "merge_dm_rids",
            [],
            NDRSignedLongField("", 0),
            size_is=lambda pkt: pkt.numRids,
        ),
    ]


class DiskMerge_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class ReAttachDisk_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("diskId", 0),
        NDRSignedLongField("diskLastKnownState", 0),
    ]


class ReAttachDisk_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class ReplaceRaid5Column_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
        NDRSignedLongField("newDiskId", 0),
        NDRSignedLongField("diskLastKnownState", 0),
    ]


class ReplaceRaid5Column_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class RestartVolume_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedLongField("volumeLastKnownState", 0),
    ]


class RestartVolume_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class GetEncapsulateDiskInfoEx_Request(NDRPacket):
    fields_desc = [
        NDRIntField("diskCount", None, size_of="diskSpecList"),
        NDRConfPacketListField(
            "diskSpecList", [], diskspec, size_is=lambda pkt: pkt.diskCount
        ),
    ]


class GetEncapsulateDiskInfoEx_Response(NDRPacket):
    fields_desc = [
        NDRIntField("encapInfoFlags", 0),
        NDRIntField("affectedDiskCount", None, size_of="affectedDiskFlags"),
        NDRConfPacketListField(
            "affectedDiskList",
            [],
            diskinfoex,
            size_is=lambda pkt: pkt.affectedDiskCount,
            ptr_pack=True,
        ),
        NDRConfFieldListField(
            "affectedDiskFlags",
            [],
            NDRIntField("", 0),
            size_is=lambda pkt: pkt.affectedDiskCount,
            ptr_pack=True,
        ),
        NDRIntField("affectedVolumeCount", None, size_of="affectedVolumeList"),
        NDRConfPacketListField(
            "affectedVolumeList",
            [],
            VOLUME_INFO,
            size_is=lambda pkt: pkt.affectedVolumeCount,
            ptr_pack=True,
        ),
        NDRIntField("affectedRegionCount", None, size_of="affectedRegionList"),
        NDRConfPacketListField(
            "affectedRegionList",
            [],
            regioninfoex,
            size_is=lambda pkt: pkt.affectedRegionCount,
            ptr_pack=True,
        ),
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class EncapsulateDiskEx_Request(NDRPacket):
    fields_desc = [
        NDRIntField("affectedDiskCount", None, size_of="affectedDiskList"),
        NDRConfPacketListField(
            "affectedDiskList",
            [],
            diskinfoex,
            size_is=lambda pkt: pkt.affectedDiskCount,
        ),
        NDRIntField("affectedVolumeCount", None, size_of="affectedVolumeList"),
        NDRConfPacketListField(
            "affectedVolumeList",
            [],
            VOLUME_INFO,
            size_is=lambda pkt: pkt.affectedVolumeCount,
        ),
        NDRIntField("affectedRegionCount", None, size_of="affectedRegionList"),
        NDRConfPacketListField(
            "affectedRegionList",
            [],
            regioninfoex,
            size_is=lambda pkt: pkt.affectedRegionCount,
        ),
    ]


class EncapsulateDiskEx_Response(NDRPacket):
    fields_desc = [
        NDRIntField("encapInfoFlags", 0),
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class QueryChangePartitionNumbers_Request(NDRPacket):
    fields_desc = []


class QueryChangePartitionNumbers_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("oldPartitionNumber", 0),
        NDRSignedIntField("newPartitionNumber", 0),
        NDRIntField("status", 0),
    ]


class DeletePartitionNumberInfoFromRegistry_Request(NDRPacket):
    fields_desc = []


class DeletePartitionNumberInfoFromRegistry_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SetDontShow_Request(NDRPacket):
    fields_desc = [NDRIntField("bSetNoShow", 0)]


class SetDontShow_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetDontShow_Request(NDRPacket):
    fields_desc = []


class GetDontShow_Response(NDRPacket):
    fields_desc = [NDRIntField("bGetNoShow", 0), NDRIntField("status", 0)]


class Reserved0_Request(NDRPacket):
    fields_desc = []


class Reserved0_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Reserved1_Request(NDRPacket):
    fields_desc = []


class Reserved1_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Reserved2_Request(NDRPacket):
    fields_desc = []


class Reserved2_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Reserved3_Request(NDRPacket):
    fields_desc = []


class Reserved3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Reserved4_Request(NDRPacket):
    fields_desc = []


class Reserved4_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Reserved5_Request(NDRPacket):
    fields_desc = []


class Reserved5_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Reserved6_Request(NDRPacket):
    fields_desc = []


class Reserved6_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Reserved7_Request(NDRPacket):
    fields_desc = []


class Reserved7_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EnumTasks_Request(NDRPacket):
    fields_desc = [NDRIntField("taskCount", 0)]


class EnumTasks_Response(NDRPacket):
    fields_desc = [
        NDRIntField("taskCount", None, size_of="taskList"),
        NDRConfPacketListField(
            "taskList", [], taskinfo, size_is=lambda pkt: pkt.taskCount, ptr_pack=True
        ),
        NDRIntField("status", 0),
    ]


class GetTaskDetail_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("id", 0),
        NDRPacketField("tinfo", taskinfo(), taskinfo),
    ]


class GetTaskDetail_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("tinfo", taskinfo(), taskinfo),
        NDRIntField("status", 0),
    ]


class AbortTask_Request(NDRPacket):
    fields_desc = [NDRSignedLongField("id", 0)]


class AbortTask_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class HrGetErrorData_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("hr", 0), NDRIntField("dwFlags", 0)]


class HrGetErrorData_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pdwStoredFlags", 0),
        NDRSignedIntField("pcszw", None, size_of="prgszw"),
        NDRConfVarStrLenFieldUtf16("prgszw", "", size_is=lambda pkt: pkt.pcszw),
        NDRIntField("status", 0),
    ]


class MInterfacePointer(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["abData"]
    fields_desc = [
        NDRIntField("ulCntData", None, size_of="abData"),
        NDRConfStrLenField(
            "abData", "", size_is=lambda pkt: pkt.ulCntData, conformant_in_struct=True
        ),
    ]


class Initialize_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("notificationInterface", MInterfacePointer(), MInterfacePointer),
        NDRIntField("cRemote", 0),
    ]


class Initialize_Response(NDRPacket):
    fields_desc = [
        NDRIntField("ulIDLVersion", 0),
        NDRIntField("pdwFlags", 0),
        NDRSignedLongField("clientId", 0),
        NDRIntField("status", 0),
    ]


class Uninitialize_Request(NDRPacket):
    fields_desc = []


class Uninitialize_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Refresh_Request(NDRPacket):
    fields_desc = []


class Refresh_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RescanDisks_Request(NDRPacket):
    fields_desc = []


class RescanDisks_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RefreshFileSys_Request(NDRPacket):
    fields_desc = []


class RefreshFileSys_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SecureSystemPartition_Request(NDRPacket):
    fields_desc = []


class SecureSystemPartition_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ShutDownSystem_Request(NDRPacket):
    fields_desc = []


class ShutDownSystem_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class countedstring(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRSignedLongField("sourceId", 0),
        NDRSignedLongField("targetId", 0),
        NDRSignedIntField("cchString", None, size_of="sstring"),
        NDRFullPointerField(
            NDRConfStrLenFieldUtf16("sstring", "", size_is=lambda pkt: pkt.cchString),
            deferred=True,
        ),
    ]


class EnumAccessPath_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("lCount", 0)]


class EnumAccessPath_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("lCount", None, size_of="paths"),
        NDRConfPacketListField(
            "paths", [], countedstring, size_is=lambda pkt: pkt.lCount, ptr_pack=True
        ),
        NDRIntField("status", 0),
    ]


class EnumAccessPathForVolume_Request(NDRPacket):
    fields_desc = [NDRSignedLongField("VolumeId", 0), NDRSignedIntField("lCount", 0)]


class EnumAccessPathForVolume_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("lCount", None, size_of="paths"),
        NDRConfPacketListField(
            "paths", [], countedstring, size_is=lambda pkt: pkt.lCount, ptr_pack=True
        ),
        NDRIntField("status", 0),
    ]


class AddAccessPath_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("cch_path", None, size_of="path"),
        NDRConfStrLenFieldUtf16("path", "", size_is=lambda pkt: pkt.cch_path),
        NDRSignedLongField("targetId", 0),
    ]


class AddAccessPath_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteAccessPath_Request(NDRPacket):
    fields_desc = [
        NDRSignedLongField("volumeId", 0),
        NDRSignedIntField("cch_path", None, size_of="path"),
        NDRConfStrLenFieldUtf16("path", "", size_is=lambda pkt: pkt.cch_path),
    ]


class DeleteAccessPath_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IVOLUMECLIENT3_OPNUMS = {
    0: DceRpcOp(EnumDisksEx_Request, EnumDisksEx_Response),
    1: DceRpcOp(EnumDiskRegionsEx_Request, EnumDiskRegionsEx_Response),
    2: DceRpcOp(CreatePartition_Request, CreatePartition_Response),
    3: DceRpcOp(
        CreatePartitionAssignAndFormat_Request, CreatePartitionAssignAndFormat_Response
    ),
    4: DceRpcOp(
        CreatePartitionAssignAndFormatEx_Request,
        CreatePartitionAssignAndFormatEx_Response,
    ),
    5: DceRpcOp(DeletePartition_Request, DeletePartition_Response),
    6: DceRpcOp(InitializeDiskStyle_Request, InitializeDiskStyle_Response),
    7: DceRpcOp(MarkActivePartition_Request, MarkActivePartition_Response),
    8: DceRpcOp(Eject_Request, Eject_Response),
    9: DceRpcOp(Reserved_Opnum12_Request, Reserved_Opnum12_Response),
    10: DceRpcOp(FTEnumVolumes_Request, FTEnumVolumes_Response),
    11: DceRpcOp(FTEnumLogicalDiskMembers_Request, FTEnumLogicalDiskMembers_Response),
    12: DceRpcOp(FTDeleteVolume_Request, FTDeleteVolume_Response),
    13: DceRpcOp(FTBreakMirror_Request, FTBreakMirror_Response),
    14: DceRpcOp(FTResyncMirror_Request, FTResyncMirror_Response),
    15: DceRpcOp(FTRegenerateParityStripe_Request, FTRegenerateParityStripe_Response),
    16: DceRpcOp(FTReplaceMirrorPartition_Request, FTReplaceMirrorPartition_Response),
    17: DceRpcOp(
        FTReplaceParityStripePartition_Request, FTReplaceParityStripePartition_Response
    ),
    18: DceRpcOp(EnumDriveLetters_Request, EnumDriveLetters_Response),
    19: DceRpcOp(AssignDriveLetter_Request, AssignDriveLetter_Response),
    20: DceRpcOp(FreeDriveLetter_Request, FreeDriveLetter_Response),
    21: DceRpcOp(EnumLocalFileSystems_Request, EnumLocalFileSystems_Response),
    22: DceRpcOp(GetInstalledFileSystems_Request, GetInstalledFileSystems_Response),
    23: DceRpcOp(Format_Request, Format_Response),
    24: DceRpcOp(EnumVolumes_Request, EnumVolumes_Response),
    25: DceRpcOp(EnumVolumeMembers_Request, EnumVolumeMembers_Response),
    26: DceRpcOp(CreateVolume_Request, CreateVolume_Response),
    27: DceRpcOp(
        CreateVolumeAssignAndFormat_Request, CreateVolumeAssignAndFormat_Response
    ),
    28: DceRpcOp(
        CreateVolumeAssignAndFormatEx_Request, CreateVolumeAssignAndFormatEx_Response
    ),
    29: DceRpcOp(GetVolumeMountName_Request, GetVolumeMountName_Response),
    30: DceRpcOp(GrowVolume_Request, GrowVolume_Response),
    31: DceRpcOp(DeleteVolume_Request, DeleteVolume_Response),
    32: DceRpcOp(CreatePartitionsForVolume_Request, CreatePartitionsForVolume_Response),
    33: DceRpcOp(DeletePartitionsForVolume_Request, DeletePartitionsForVolume_Response),
    34: DceRpcOp(GetMaxAdjustedFreeSpace_Request, GetMaxAdjustedFreeSpace_Response),
    35: DceRpcOp(AddMirror_Request, AddMirror_Response),
    36: DceRpcOp(RemoveMirror_Request, RemoveMirror_Response),
    37: DceRpcOp(SplitMirror_Request, SplitMirror_Response),
    38: DceRpcOp(InitializeDiskEx_Request, InitializeDiskEx_Response),
    39: DceRpcOp(UninitializeDisk_Request, UninitializeDisk_Response),
    40: DceRpcOp(ReConnectDisk_Request, ReConnectDisk_Response),
    41: DceRpcOp(ImportDiskGroup_Request, ImportDiskGroup_Response),
    42: DceRpcOp(DiskMergeQuery_Request, DiskMergeQuery_Response),
    43: DceRpcOp(DiskMerge_Request, DiskMerge_Response),
    44: DceRpcOp(ReAttachDisk_Request, ReAttachDisk_Response),
    45: DceRpcOp(ReplaceRaid5Column_Request, ReplaceRaid5Column_Response),
    46: DceRpcOp(RestartVolume_Request, RestartVolume_Response),
    47: DceRpcOp(GetEncapsulateDiskInfoEx_Request, GetEncapsulateDiskInfoEx_Response),
    48: DceRpcOp(EncapsulateDiskEx_Request, EncapsulateDiskEx_Response),
    49: DceRpcOp(
        QueryChangePartitionNumbers_Request, QueryChangePartitionNumbers_Response
    ),
    50: DceRpcOp(
        DeletePartitionNumberInfoFromRegistry_Request,
        DeletePartitionNumberInfoFromRegistry_Response,
    ),
    51: DceRpcOp(SetDontShow_Request, SetDontShow_Response),
    52: DceRpcOp(GetDontShow_Request, GetDontShow_Response),
    53: DceRpcOp(Reserved0_Request, Reserved0_Response),
    54: DceRpcOp(Reserved1_Request, Reserved1_Response),
    55: DceRpcOp(Reserved2_Request, Reserved2_Response),
    56: DceRpcOp(Reserved3_Request, Reserved3_Response),
    57: DceRpcOp(Reserved4_Request, Reserved4_Response),
    58: DceRpcOp(Reserved5_Request, Reserved5_Response),
    59: DceRpcOp(Reserved6_Request, Reserved6_Response),
    60: DceRpcOp(Reserved7_Request, Reserved7_Response),
    61: DceRpcOp(EnumTasks_Request, EnumTasks_Response),
    62: DceRpcOp(GetTaskDetail_Request, GetTaskDetail_Response),
    63: DceRpcOp(AbortTask_Request, AbortTask_Response),
    64: DceRpcOp(HrGetErrorData_Request, HrGetErrorData_Response),
    65: DceRpcOp(Initialize_Request, Initialize_Response),
    66: DceRpcOp(Uninitialize_Request, Uninitialize_Response),
    67: DceRpcOp(Refresh_Request, Refresh_Response),
    68: DceRpcOp(RescanDisks_Request, RescanDisks_Response),
    69: DceRpcOp(RefreshFileSys_Request, RefreshFileSys_Response),
    70: DceRpcOp(SecureSystemPartition_Request, SecureSystemPartition_Response),
    71: DceRpcOp(ShutDownSystem_Request, ShutDownSystem_Response),
    72: DceRpcOp(EnumAccessPath_Request, EnumAccessPath_Response),
    73: DceRpcOp(EnumAccessPathForVolume_Request, EnumAccessPathForVolume_Response),
    74: DceRpcOp(AddAccessPath_Request, AddAccessPath_Response),
    75: DceRpcOp(DeleteAccessPath_Request, DeleteAccessPath_Response),
}
register_com_interface(
    name="IVolumeClient3",
    uuid=uuid.UUID("135698D2-3A37-4d26-99DF-E2BB6AE3AC61"),
    opnums=IVOLUMECLIENT3_OPNUMS,
)


class RefreshEx_Request(NDRPacket):
    fields_desc = []


class RefreshEx_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetVolumeDeviceName_Request(NDRPacket):
    fields_desc = [NDRSignedLongField("_volumeId", 0)]


class GetVolumeDeviceName_Response(NDRPacket):
    fields_desc = [
        NDRIntField("cchVolumeDevice", None, size_of="pwszVolumeDevice"),
        NDRConfStrLenFieldUtf16(
            "pwszVolumeDevice", "", size_is=lambda pkt: pkt.cchVolumeDevice
        ),
        NDRIntField("status", 0),
    ]


IVOLUMECLIENT4_OPNUMS = {
    0: DceRpcOp(RefreshEx_Request, RefreshEx_Response),
    1: DceRpcOp(GetVolumeDeviceName_Request, GetVolumeDeviceName_Response),
}
register_com_interface(
    name="IVolumeClient4",
    uuid=uuid.UUID("DEB01010-3A37-4d26-99DF-E2BB6AE3AC61"),
    opnums=IVOLUMECLIENT4_OPNUMS,
)



### ms-oaut.idl
"""
RPC definitions for the following interfaces:
- IDispatch (v0.0): 00020400-0000-0000-C000-000000000046
- ITypeLib (v0.0): 00020402-0000-0000-C000-000000000046
- ITypeInfo (v0.0): 00020401-0000-0000-C000-000000000046
- IEnumVARIANT (v0.0): 00020404-0000-0000-C000-000000000046
- ITypeComp (v0.0): 00020403-0000-0000-C000-000000000046
- ITypeInfo2 (v0.0): 00020412-0000-0000-C000-000000000046
- ITypeLib2 (v0.0): 00020411-0000-0000-C000-000000000046
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import PacketListField, StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfFieldListField,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarFieldListField,
    NDRConfVarPacketListField,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRInt3264Field,
    NDRIntEnumField,
    NDRIntField,
    NDRPacketField,
    NDRRecursiveField,
    NDRRefEmbPointerField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    NDRSignedShortField,
    NDRUnionField,
    register_com_interface,
)


class GetTypeInfoCount_Request(NDRPacket):
    fields_desc = []


class GetTypeInfoCount_Response(NDRPacket):
    fields_desc = [NDRIntField("pctinfo", 0), NDRIntField("status", 0)]


class MInterfacePointer(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["abData"]
    fields_desc = [
        NDRIntField("ulCntData", None, size_of="abData"),
        NDRConfStrLenField(
            "abData", "", size_is=lambda pkt: pkt.ulCntData, conformant_in_struct=True
        ),
    ]


class GetTypeInfo_Request(NDRPacket):
    fields_desc = [NDRIntField("iTInfo", 0), NDRIntField("lcid", 0)]


class GetTypeInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppTInfo", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class GetIDsOfNames_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("riid", GUID(), GUID),
        NDRConfVarStrLenFieldUtf16("rgszNames", "", size_is=lambda pkt: pkt.cNames),
        NDRIntField("cNames", None, size_of="rgszNames"),
        NDRIntField("lcid", 0),
    ]


class GetIDsOfNames_Response(NDRPacket):
    fields_desc = [
        NDRConfFieldListField(
            "rgDispId", [], NDRSignedIntField("", 0), size_is=lambda pkt: pkt.cNames
        ),
        NDRIntField("status", 0),
    ]


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_R4 = 4
    VT_R8 = 5
    VT_CY = 6
    VT_DATE = 7
    VT_BSTR = 8
    VT_DISPATCH = 9
    VT_ERROR = 10
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_UNKNOWN = 13
    VT_DECIMAL = 14
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_INT = 22
    VT_UINT = 23
    VT_VOID = 24
    VT_HRESULT = 25
    VT_PTR = 26
    VT_SAFEARRAY = 27
    VT_CARRAY = 28
    VT_USERDEFINED = 29
    VT_LPSTR = 30
    VT_LPWSTR = 31
    VT_RECORD = 36
    VT_INT_PTR = 37
    VT_UINT_PTR = 38
    VT_ARRAY = 8192
    VT_BYREF = 16384


class CURRENCY(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("int64", 0)]


class FLAGGED_WORD_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["asData"]
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRIntField("clSize", None, size_of="asData"),
        NDRConfStrLenFieldUtf16(
            "asData", "", size_is=lambda pkt: pkt.clSize, conformant_in_struct=True
        ),
    ]


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_R4 = 4
    VT_R8 = 5
    VT_CY = 6
    VT_DATE = 7
    VT_BSTR = 8
    VT_DISPATCH = 9
    VT_ERROR = 10
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_UNKNOWN = 13
    VT_DECIMAL = 14
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_INT = 22
    VT_UINT = 23
    VT_VOID = 24
    VT_HRESULT = 25
    VT_PTR = 26
    VT_SAFEARRAY = 27
    VT_CARRAY = 28
    VT_USERDEFINED = 29
    VT_LPSTR = 30
    VT_LPWSTR = 31
    VT_RECORD = 36
    VT_INT_PTR = 37
    VT_UINT_PTR = 38
    VT_ARRAY = 8192
    VT_BYREF = 16384


class SF_TYPE(IntEnum):
    SF_ERROR = VARENUM.VT_ERROR
    SF_I1 = VARENUM.VT_I1
    SF_I2 = VARENUM.VT_I2
    SF_I4 = VARENUM.VT_I4
    SF_I8 = VARENUM.VT_I8
    SF_BSTR = VARENUM.VT_BSTR
    SF_UNKNOWN = VARENUM.VT_UNKNOWN
    SF_DISPATCH = VARENUM.VT_DISPATCH
    SF_VARIANT = VARENUM.VT_VARIANT
    SF_RECORD = VARENUM.VT_RECORD
    SF_HAVEIID = VARENUM.VT_UNKNOWN | 32768


class SAFEARR_BSTR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="aBstr"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "aBstr", [], FLAGGED_WORD_BLOB, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class SAFEARR_UNKNOWN(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="apUnknown"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "apUnknown", [], MInterfacePointer, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class SAFEARR_DISPATCH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="apDispatch"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "apDispatch", [], MInterfacePointer, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class wireVARIANTStr(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("clSize", 0),
        NDRIntField("rpcReserved", 0),
        NDRShortField("vt", 0),
        NDRShortField("wReserved1", 0),
        NDRShortField("wReserved2", 0),
        NDRShortField("wReserved3", 0),
        NDRRecursiveField("_varUnion"),
    ]


class DISPPARAMS(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfPacketListField(
                "rgvarg",
                [],
                wireVARIANTStr,
                size_is=lambda pkt: pkt.cArgs,
                ptr_pack=True,
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRConfFieldListField(
                "rgdispidNamedArgs",
                [],
                NDRSignedIntField("rgdispidNamedArgs", 0),
                size_is=lambda pkt: pkt.cNamedArgs,
            ),
            deferred=True,
        ),
        NDRIntField("cArgs", None, size_of="rgvarg"),
        NDRIntField("cNamedArgs", None, size_of="rgdispidNamedArgs"),
    ]


class EXCEPINFO(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("wCode", 0),
        NDRShortField("wReserved", 0),
        NDRFullPointerField(
            NDRPacketField("bstrSource", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRPacketField("bstrDescription", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRPacketField("bstrHelpFile", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
            deferred=True,
        ),
        NDRIntField("dwHelpContext", 0),
        NDRInt3264Field("pvReserved", 0),
        NDRInt3264Field("pfnDeferredFillIn", 0),
        NDRSignedIntField("scode", 0),
    ]


class Invoke_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("dispIdMember", 0),
        NDRPacketField("riid", GUID(), GUID),
        NDRIntField("lcid", 0),
        NDRIntField("dwFlags", 0),
        NDRPacketField("pDispParams", DISPPARAMS(), DISPPARAMS),
        NDRIntField("cVarRef", None, size_of="rgVarRef"),
        NDRConfFieldListField(
            "rgVarRefIdx", [], NDRIntField("", 0), size_is=lambda pkt: pkt.cVarRef
        ),
        NDRConfPacketListField(
            "rgVarRef",
            [],
            wireVARIANTStr,
            size_is=lambda pkt: pkt.cVarRef,
            ptr_pack=True,
        ),
    ]


class Invoke_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pVarResult", wireVARIANTStr(), wireVARIANTStr)
        ),
        NDRPacketField("pExcepInfo", EXCEPINFO(), EXCEPINFO),
        NDRIntField("pArgErr", 0),
        NDRConfPacketListField(
            "rgVarRef",
            [],
            wireVARIANTStr,
            size_is=lambda pkt: pkt.cVarRef,
            ptr_pack=True,
        ),
        NDRIntField("status", 0),
    ]


IDISPATCH_OPNUMS = {
    0: DceRpcOp(GetTypeInfoCount_Request, GetTypeInfoCount_Response),
    1: DceRpcOp(GetTypeInfo_Request, GetTypeInfo_Response),
    2: DceRpcOp(GetIDsOfNames_Request, GetIDsOfNames_Response),
    3: DceRpcOp(Invoke_Request, Invoke_Response),
}
register_com_interface(
    name="IDispatch",
    uuid=uuid.UUID("00020400-0000-0000-C000-000000000046"),
    opnums=IDISPATCH_OPNUMS,
)


class GetTypeInfoCount_Request(NDRPacket):
    fields_desc = []


class GetTypeInfoCount_Response(NDRPacket):
    fields_desc = [NDRIntField("pcTInfo", 0), NDRIntField("status", 0)]


class GetTypeInfo_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0)]


class GetTypeInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppTInfo", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class TYPEKIND(IntEnum):
    TKIND_ENUM = 0
    TKIND_RECORD = 1
    TKIND_MODULE = 2
    TKIND_INTERFACE = 3
    TKIND_DISPATCH = 4
    TKIND_COCLASS = 5
    TKIND_ALIAS = 6
    TKIND_UNION = 7


class GetTypeInfoType_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0)]


class GetTypeInfoType_Response(NDRPacket):
    fields_desc = [NDRIntEnumField("pTKind", 0, TYPEKIND), NDRIntField("status", 0)]


class GetTypeInfoOfGuid_Request(NDRPacket):
    fields_desc = [NDRPacketField("guid", GUID(), GUID)]


class GetTypeInfoOfGuid_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppTInfo", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class SYSKIND(IntEnum):
    SYS_WIN32 = 1
    SYS_WIN64 = 3


class LPTLIBATTR(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("guid", GUID(), GUID),
        NDRIntField("lcid", 0),
        NDRIntEnumField("syskind", 0, SYSKIND),
        NDRShortField("wMajorVerNum", 0),
        NDRShortField("wMinorVerNum", 0),
        NDRShortField("wLibFlags", 0),
    ]


class GetLibAttr_Request(NDRPacket):
    fields_desc = []


class GetLibAttr_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ppTLibAttr", LPTLIBATTR(), LPTLIBATTR)),
        NDRIntField("pReserved", 0),
        NDRIntField("status", 0),
    ]


class GetTypeComp_Request(NDRPacket):
    fields_desc = []


class GetTypeComp_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppTComp", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class GetDocumentation_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("index", 0), NDRIntField("refPtrFlags", 0)]


class GetDocumentation_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pBstrName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRFullPointerField(
            NDRPacketField("pBstrDocString", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("pdwHelpContext", 0),
        NDRFullPointerField(
            NDRPacketField("pBstrHelpFile", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class IsName_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("szNameBuf", ""),
        NDRIntField("lHashVal", 0),
    ]


class IsName_Response(NDRPacket):
    fields_desc = [
        NDRSignedIntField("pfName", 0),
        NDRFullPointerField(
            NDRPacketField("pBstrNameInLibrary", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class FindName_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("szNameBuf", ""),
        NDRIntField("lHashVal", 0),
        NDRShortField("pcFound", 0),
    ]


class FindName_Response(NDRPacket):
    fields_desc = [
        NDRConfVarPacketListField(
            "ppTInfo",
            [],
            MInterfacePointer,
            size_is=lambda pkt: pkt.pcFound,
            length_is=lambda pkt: pkt.pcFound,
            ptr_pack=True,
        ),
        NDRConfVarFieldListField(
            "rgMemId",
            [],
            NDRSignedIntField("", 0),
            size_is=lambda pkt: pkt.pcFound,
            length_is=lambda pkt: pkt.pcFound,
        ),
        NDRShortField("pcFound", None, size_of="rgMemId"),
        NDRFullPointerField(
            NDRPacketField("pBstrNameInLibrary", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


ITYPELIB_OPNUMS = {
    0: DceRpcOp(GetTypeInfoCount_Request, GetTypeInfoCount_Response),
    1: DceRpcOp(GetTypeInfo_Request, GetTypeInfo_Response),
    2: DceRpcOp(GetTypeInfoType_Request, GetTypeInfoType_Response),
    3: DceRpcOp(GetTypeInfoOfGuid_Request, GetTypeInfoOfGuid_Response),
    4: DceRpcOp(GetLibAttr_Request, GetLibAttr_Response),
    5: DceRpcOp(GetTypeComp_Request, GetTypeComp_Response),
    6: DceRpcOp(GetDocumentation_Request, GetDocumentation_Response),
    7: DceRpcOp(IsName_Request, IsName_Response),
    8: DceRpcOp(FindName_Request, FindName_Response),
    # 9: Opnum12NotUsedOnWire
}
register_com_interface(
    name="ITypeLib",
    uuid=uuid.UUID("00020402-0000-0000-C000-000000000046"),
    opnums=ITYPELIB_OPNUMS,
)


class TYPEDESC(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [NDRRecursiveField("_tdUnion"), NDRShortField("vt", 0)]


class LPTYPEATTR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("guid", GUID(), GUID),
        NDRIntField("lcid", 0),
        NDRIntField("dwReserved1", 0),
        NDRIntField("dwReserved2", 0),
        NDRIntField("dwReserved3", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("lpstrReserved4", ""), deferred=True
        ),
        NDRIntField("cbSizeInstance", 0),
        NDRIntEnumField("typekind", 0, TYPEKIND),
        NDRShortField("cFuncs", 0),
        NDRShortField("cVars", 0),
        NDRShortField("cImplTypes", 0),
        NDRShortField("cbSizeVft", 0),
        NDRShortField("cbAlignment", 0),
        NDRShortField("wTypeFlags", 0),
        NDRShortField("wMajorVerNum", 0),
        NDRShortField("wMinorVerNum", 0),
        NDRPacketField("tdescAlias", TYPEDESC(), TYPEDESC),
        NDRIntField("dwReserved5", 0),
        NDRShortField("wReserved6", 0),
    ]


class GetTypeAttr_Request(NDRPacket):
    fields_desc = []


class GetTypeAttr_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ppTypeAttr", LPTYPEATTR(), LPTYPEATTR)),
        NDRIntField("pReserved", 0),
        NDRIntField("status", 0),
    ]


class GetTypeComp_Request(NDRPacket):
    fields_desc = []


class GetTypeComp_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppTComp", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class PARAMDESCEX(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRFullPointerField(
            NDRPacketField("varDefaultValue", wireVARIANTStr(), wireVARIANTStr),
            deferred=True,
        ),
    ]


class PARAMDESC(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pparamdescex", PARAMDESCEX(), PARAMDESCEX), deferred=True
        ),
        NDRShortField("wParamFlags", 0),
    ]


class ELEMDESC(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("tdesc", TYPEDESC(), TYPEDESC),
        NDRPacketField("paramdesc", PARAMDESC(), PARAMDESC),
    ]


class FUNCKIND(IntEnum):
    FUNC_PUREVIRTUAL = 1
    FUNC_STATIC = 3
    FUNC_DISPATCH = 4


class INVOKEKIND(IntEnum):
    INVOKE_FUNC = 1
    INVOKE_PROPERTYGET = 2
    INVOKE_PROPERTYPUT = 4
    INVOKE_PROPERTYPUTREF = 8


class CALLCONV(IntEnum):
    CC_CDECL = 1
    CC_PASCAL = 2
    CC_STDCALL = 4


class LPFUNCDESC(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRSignedIntField("memid", 0),
        NDRFullPointerField(
            NDRConfFieldListField(
                "lReserved1",
                [],
                NDRSignedIntField("lReserved1", 0),
                size_is=lambda pkt: pkt.cReserved2,
            ),
            deferred=True,
        ),
        NDRFullPointerField(
            NDRConfPacketListField(
                "lprgelemdescParam", [], ELEMDESC, size_is=lambda pkt: pkt.cParams
            ),
            deferred=True,
        ),
        NDRIntEnumField("funckind", 0, FUNCKIND),
        NDRIntEnumField("invkind", 0, INVOKEKIND),
        NDRIntEnumField("callconv", 0, CALLCONV),
        NDRSignedShortField("cParams", None, size_of="lprgelemdescParam"),
        NDRSignedShortField("cParamsOpt", 0),
        NDRSignedShortField("oVft", 0),
        NDRSignedShortField("cReserved2", None, size_of="lReserved1"),
        NDRPacketField("elemdescFunc", ELEMDESC(), ELEMDESC),
        NDRShortField("wFuncFlags", 0),
    ]


class GetFuncDesc_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0)]


class GetFuncDesc_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ppFuncDesc", LPFUNCDESC(), LPFUNCDESC)),
        NDRIntField("pReserved", 0),
        NDRIntField("status", 0),
    ]


class VARKIND(IntEnum):
    VAR_PERINSTANCE = 0
    VAR_STATIC = VAR_PERINSTANCE + 1
    VAR_CONST = VAR_STATIC + 1
    VAR_DISPATCH = VAR_CONST + 1


class LPVARDESC(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRSignedIntField("memid", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("lpstrReserved", ""), deferred=True
        ),
        NDRUnionField(
            [
                (
                    NDRIntField("_vdUnion", 0),
                    (
                        (
                            lambda pkt: getattr(pkt, "varkind", None)
                            in [
                                VARKIND.VAR_PERINSTANCE,
                                VARKIND.VAR_DISPATCH,
                                VARKIND.VAR_STATIC,
                            ]
                        ),
                        (
                            lambda _, val: val.tag
                            in [
                                VARKIND.VAR_PERINSTANCE,
                                VARKIND.VAR_DISPATCH,
                                VARKIND.VAR_STATIC,
                            ]
                        ),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRFullPointerField(
                            NDRPacketField(
                                "_vdUnion", wireVARIANTStr(), wireVARIANTStr
                            ),
                            deferred=True,
                        ),
                        deferred=True,
                    ),
                    (
                        (
                            lambda pkt: getattr(pkt, "varkind", None)
                            == VARKIND.VAR_CONST
                        ),
                        (lambda _, val: val.tag == VARKIND.VAR_CONST),
                    ),
                ),
            ],
            StrFixedLenField("_vdUnion", "", length=0),
            align=(4, 8),
            switch_fmt=("I", "I"),
        ),
        NDRPacketField("elemdescVar", ELEMDESC(), ELEMDESC),
        NDRShortField("wVarFlags", 0),
        NDRIntEnumField("varkind", 0, VARKIND),
    ]


class GetVarDesc_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0)]


class GetVarDesc_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRPacketField("ppVarDesc", LPVARDESC(), LPVARDESC)),
        NDRIntField("pReserved", 0),
        NDRIntField("status", 0),
    ]


class GetNames_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("memid", 0), NDRIntField("cMaxNames", 0)]


class GetNames_Response(NDRPacket):
    fields_desc = [
        NDRConfVarPacketListField(
            "rgBstrNames",
            [],
            FLAGGED_WORD_BLOB,
            size_is=lambda pkt: pkt.cMaxNames,
            length_is=lambda pkt: pkt.pcNames,
            ptr_pack=True,
        ),
        NDRIntField("pcNames", None, size_of="rgBstrNames"),
        NDRIntField("status", 0),
    ]


class GetRefTypeOfImplType_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0)]


class GetRefTypeOfImplType_Response(NDRPacket):
    fields_desc = [NDRIntField("pRefType", 0), NDRIntField("status", 0)]


class GetImplTypeFlags_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0)]


class GetImplTypeFlags_Response(NDRPacket):
    fields_desc = [NDRSignedIntField("pImplTypeFlags", 0), NDRIntField("status", 0)]


class GetDocumentation_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("memid", 0), NDRIntField("refPtrFlags", 0)]


class GetDocumentation_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pBstrName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRFullPointerField(
            NDRPacketField("pBstrDocString", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("pdwHelpContext", 0),
        NDRFullPointerField(
            NDRPacketField("pBstrHelpFile", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GetDllEntry_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("memid", 0),
        NDRIntEnumField("invKind", 0, INVOKEKIND),
        NDRIntField("refPtrFlags", 0),
    ]


class GetDllEntry_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pBstrDllName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRFullPointerField(
            NDRPacketField("pBstrName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRShortField("pwOrdinal", 0),
        NDRIntField("status", 0),
    ]


class GetRefTypeInfo_Request(NDRPacket):
    fields_desc = [NDRIntField("hRefType", 0)]


class GetRefTypeInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppTInfo", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class CreateInstance_Request(NDRPacket):
    fields_desc = [NDRPacketField("riid", GUID(), GUID)]


class CreateInstance_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppvObj", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class GetMops_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("memid", 0)]


class GetMops_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pBstrMops", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GetContainingTypeLib_Request(NDRPacket):
    fields_desc = []


class GetContainingTypeLib_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppTLib", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("pIndex", 0),
        NDRIntField("status", 0),
    ]


ITYPEINFO_OPNUMS = {
    0: DceRpcOp(GetTypeAttr_Request, GetTypeAttr_Response),
    1: DceRpcOp(GetTypeComp_Request, GetTypeComp_Response),
    2: DceRpcOp(GetFuncDesc_Request, GetFuncDesc_Response),
    3: DceRpcOp(GetVarDesc_Request, GetVarDesc_Response),
    4: DceRpcOp(GetNames_Request, GetNames_Response),
    5: DceRpcOp(GetRefTypeOfImplType_Request, GetRefTypeOfImplType_Response),
    6: DceRpcOp(GetImplTypeFlags_Request, GetImplTypeFlags_Response),
    # 7: Opnum10NotUsedOnWire,
    # 8: Opnum11NotUsedOnWire,
    9: DceRpcOp(GetDocumentation_Request, GetDocumentation_Response),
    10: DceRpcOp(GetDllEntry_Request, GetDllEntry_Response),
    11: DceRpcOp(GetRefTypeInfo_Request, GetRefTypeInfo_Response),
    # 12: Opnum15NotUsedOnWire,
    13: DceRpcOp(CreateInstance_Request, CreateInstance_Response),
    14: DceRpcOp(GetMops_Request, GetMops_Response),
    15: DceRpcOp(GetContainingTypeLib_Request, GetContainingTypeLib_Response),
    # 16: Opnum19NotUsedOnWire,
    # 17: Opnum20NotUsedOnWire,
    # 18: Opnum21NotUsedOnWire
}
register_com_interface(
    name="ITypeInfo",
    uuid=uuid.UUID("00020401-0000-0000-C000-000000000046"),
    opnums=ITYPEINFO_OPNUMS,
)


class Next_Request(NDRPacket):
    fields_desc = [NDRIntField("celt", 0)]


class Next_Response(NDRPacket):
    fields_desc = [
        NDRConfVarPacketListField(
            "rgVar",
            [],
            wireVARIANTStr,
            size_is=lambda pkt: pkt.celt,
            length_is=lambda pkt: pkt.pCeltFetched,
            ptr_pack=True,
        ),
        NDRIntField("pCeltFetched", None, size_of="rgVar"),
        NDRIntField("status", 0),
    ]


class Skip_Request(NDRPacket):
    fields_desc = [NDRIntField("celt", 0)]


class Skip_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Reset_Request(NDRPacket):
    fields_desc = []


class Reset_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Clone_Request(NDRPacket):
    fields_desc = []


class Clone_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppEnum", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


IENUMVARIANT_OPNUMS = {
    0: DceRpcOp(Next_Request, Next_Response),
    1: DceRpcOp(Skip_Request, Skip_Response),
    2: DceRpcOp(Reset_Request, Reset_Response),
    3: DceRpcOp(Clone_Request, Clone_Response),
}
register_com_interface(
    name="IEnumVARIANT",
    uuid=uuid.UUID("00020404-0000-0000-C000-000000000046"),
    opnums=IENUMVARIANT_OPNUMS,
)


class DESCKIND(IntEnum):
    DESCKIND_NONE = 0
    DESCKIND_FUNCDESC = 1
    DESCKIND_VARDESC = 2
    DESCKIND_TYPECOMP = 3
    DESCKIND_IMPLICITAPPOBJ = 4


class Bind_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("szName", ""),
        NDRIntField("lHashVal", 0),
        NDRShortField("wFlags", 0),
    ]


class Bind_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppTInfo", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntEnumField("pDescKind", 0, DESCKIND),
        NDRFullPointerField(NDRPacketField("ppFuncDesc", LPFUNCDESC(), LPFUNCDESC)),
        NDRFullPointerField(NDRPacketField("ppVarDesc", LPVARDESC(), LPVARDESC)),
        NDRFullPointerField(
            NDRPacketField("ppTypeComp", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("pReserved", 0),
        NDRIntField("status", 0),
    ]


class BindType_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrNullFieldUtf16("szName", ""),
        NDRIntField("lHashVal", 0),
    ]


class BindType_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppTInfo", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


ITYPECOMP_OPNUMS = {
    0: DceRpcOp(Bind_Request, Bind_Response),
    1: DceRpcOp(BindType_Request, BindType_Response),
}
register_com_interface(
    name="ITypeComp",
    uuid=uuid.UUID("00020403-0000-0000-C000-000000000046"),
    opnums=ITYPECOMP_OPNUMS,
)


class GetTypeKind_Request(NDRPacket):
    fields_desc = []


class GetTypeKind_Response(NDRPacket):
    fields_desc = [NDRIntEnumField("pTypeKind", 0, TYPEKIND), NDRIntField("status", 0)]


class GetTypeFlags_Request(NDRPacket):
    fields_desc = []


class GetTypeFlags_Response(NDRPacket):
    fields_desc = [NDRIntField("pTypeFlags", 0), NDRIntField("status", 0)]


class GetFuncIndexOfMemId_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("memid", 0),
        NDRIntEnumField("invKind", 0, INVOKEKIND),
    ]


class GetFuncIndexOfMemId_Response(NDRPacket):
    fields_desc = [NDRIntField("pFuncIndex", 0), NDRIntField("status", 0)]


class GetVarIndexOfMemId_Request(NDRPacket):
    fields_desc = [NDRSignedIntField("memid", 0)]


class GetVarIndexOfMemId_Response(NDRPacket):
    fields_desc = [NDRIntField("pVarIndex", 0), NDRIntField("status", 0)]


class GetCustData_Request(NDRPacket):
    fields_desc = [NDRPacketField("guid", GUID(), GUID)]


class GetCustData_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pVarVal", wireVARIANTStr(), wireVARIANTStr)
        ),
        NDRIntField("status", 0),
    ]


class GetFuncCustData_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0), NDRPacketField("guid", GUID(), GUID)]


class GetFuncCustData_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pVarVal", wireVARIANTStr(), wireVARIANTStr)
        ),
        NDRIntField("status", 0),
    ]


class GetParamCustData_Request(NDRPacket):
    fields_desc = [
        NDRIntField("indexFunc", 0),
        NDRIntField("indexParam", 0),
        NDRPacketField("guid", GUID(), GUID),
    ]


class GetParamCustData_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pVarVal", wireVARIANTStr(), wireVARIANTStr)
        ),
        NDRIntField("status", 0),
    ]


class GetVarCustData_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0), NDRPacketField("guid", GUID(), GUID)]


class GetVarCustData_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pVarVal", wireVARIANTStr(), wireVARIANTStr)
        ),
        NDRIntField("status", 0),
    ]


class GetImplTypeCustData_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0), NDRPacketField("guid", GUID(), GUID)]


class GetImplTypeCustData_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pVarVal", wireVARIANTStr(), wireVARIANTStr)
        ),
        NDRIntField("status", 0),
    ]


class GetDocumentation2_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("memid", 0),
        NDRIntField("lcid", 0),
        NDRIntField("refPtrFlags", 0),
    ]


class GetDocumentation2_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrHelpString", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("pdwHelpStringContext", 0),
        NDRFullPointerField(
            NDRPacketField("pbstrHelpStringDll", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class CUSTDATAITEM(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("guid", GUID(), GUID),
        NDRFullPointerField(
            NDRPacketField("varValue", wireVARIANTStr(), wireVARIANTStr), deferred=True
        ),
    ]


class CUSTDATA(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("cCustData", None, size_of="prgCustData"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "prgCustData", [], CUSTDATAITEM, size_is=lambda pkt: pkt.cCustData
            ),
            deferred=True,
        ),
    ]


class GetAllCustData_Request(NDRPacket):
    fields_desc = []


class GetAllCustData_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pCustData", CUSTDATA(), CUSTDATA),
        NDRIntField("status", 0),
    ]


class GetAllFuncCustData_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0)]


class GetAllFuncCustData_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pCustData", CUSTDATA(), CUSTDATA),
        NDRIntField("status", 0),
    ]


class GetAllParamCustData_Request(NDRPacket):
    fields_desc = [NDRIntField("indexFunc", 0), NDRIntField("indexParam", 0)]


class GetAllParamCustData_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pCustData", CUSTDATA(), CUSTDATA),
        NDRIntField("status", 0),
    ]


class GetAllVarCustData_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0)]


class GetAllVarCustData_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pCustData", CUSTDATA(), CUSTDATA),
        NDRIntField("status", 0),
    ]


class GetAllImplTypeCustData_Request(NDRPacket):
    fields_desc = [NDRIntField("index", 0)]


class GetAllImplTypeCustData_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pCustData", CUSTDATA(), CUSTDATA),
        NDRIntField("status", 0),
    ]


ITYPEINFO2_OPNUMS = {
    0: DceRpcOp(GetTypeKind_Request, GetTypeKind_Response),
    1: DceRpcOp(GetTypeFlags_Request, GetTypeFlags_Response),
    2: DceRpcOp(GetFuncIndexOfMemId_Request, GetFuncIndexOfMemId_Response),
    3: DceRpcOp(GetVarIndexOfMemId_Request, GetVarIndexOfMemId_Response),
    4: DceRpcOp(GetCustData_Request, GetCustData_Response),
    5: DceRpcOp(GetFuncCustData_Request, GetFuncCustData_Response),
    6: DceRpcOp(GetParamCustData_Request, GetParamCustData_Response),
    7: DceRpcOp(GetVarCustData_Request, GetVarCustData_Response),
    8: DceRpcOp(GetImplTypeCustData_Request, GetImplTypeCustData_Response),
    9: DceRpcOp(GetDocumentation2_Request, GetDocumentation2_Response),
    10: DceRpcOp(GetAllCustData_Request, GetAllCustData_Response),
    11: DceRpcOp(GetAllFuncCustData_Request, GetAllFuncCustData_Response),
    12: DceRpcOp(GetAllParamCustData_Request, GetAllParamCustData_Response),
    13: DceRpcOp(GetAllVarCustData_Request, GetAllVarCustData_Response),
    14: DceRpcOp(GetAllImplTypeCustData_Request, GetAllImplTypeCustData_Response),
}
register_com_interface(
    name="ITypeInfo2",
    uuid=uuid.UUID("00020412-0000-0000-C000-000000000046"),
    opnums=ITYPEINFO2_OPNUMS,
)


class GetCustData_Request(NDRPacket):
    fields_desc = [NDRPacketField("guid", GUID(), GUID)]


class GetCustData_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pVarVal", wireVARIANTStr(), wireVARIANTStr)
        ),
        NDRIntField("status", 0),
    ]


class GetLibStatistics_Request(NDRPacket):
    fields_desc = []


class GetLibStatistics_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcUniqueNames", 0),
        NDRIntField("pcchUniqueNames", 0),
        NDRIntField("status", 0),
    ]


class GetDocumentation2_Request(NDRPacket):
    fields_desc = [
        NDRSignedIntField("index", 0),
        NDRIntField("lcid", 0),
        NDRIntField("refPtrFlags", 0),
    ]


class GetDocumentation2_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pbstrHelpString", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("pdwHelpStringContext", 0),
        NDRFullPointerField(
            NDRPacketField("pbstrHelpStringDll", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GetAllCustData_Request(NDRPacket):
    fields_desc = []


class GetAllCustData_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pCustData", CUSTDATA(), CUSTDATA),
        NDRIntField("status", 0),
    ]


ITYPELIB2_OPNUMS = {
    0: DceRpcOp(GetCustData_Request, GetCustData_Response),
    1: DceRpcOp(GetLibStatistics_Request, GetLibStatistics_Response),
    2: DceRpcOp(GetDocumentation2_Request, GetDocumentation2_Response),
    3: DceRpcOp(GetAllCustData_Request, GetAllCustData_Response),
}
register_com_interface(
    name="ITypeLib2",
    uuid=uuid.UUID("00020411-0000-0000-C000-000000000046"),
    opnums=ITYPELIB2_OPNUMS,
)
